@@ -19,13 +19,34 @@
 
 """Provides the interface to the AT-SPI Registry."""
 
+__id__        = "$Id: atspi.py,v 1.46 2006/08/28 13:52:17 oserb Exp $"
+__version__   = "$Revision: 1.46 $"
+__date__      = "$Date: 2006/08/28 13:52:17 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
+
+import signal
 import time
 
+import gobject
+gobject.threads_init()
+
 import bonobo
 import ORBit
 
 ORBit.load_typelib("Accessibility")
-ORBit.CORBA.ORB_init()
+
+# We will pass "orbit-io-thread" to initialize the ORB in threaded mode.
+# This should hopefully help address bug 319652:
+#
+#   http://bugzilla.gnome.org/show_bug.cgi?id=319652
+#
+# See also:
+#
+#   http://bugzilla.gnome.org/show_bug.cgi?id=342614
+#   http://mail.gnome.org/archives/orbit-list/2005-December/msg00001.html
+#
+ORBit.CORBA.ORB_init(orb_id="orbit-io-thread")
 
 import Accessibility
 import Accessibility__POA
@@ -121,12 +142,40 @@ class Registry:
         if not self.__dict__.has_key("desktop"):
             self.desktop = self.registry.getDesktop(0)
 
+    def __blockPreventor(self):
+        """[[[TODO: HACK to attempt to prevent deadlocks.  We call time.sleep
+        here as a means to sidestep the global interpreter lock (GIL).]]]
+        """
+        if settings.gilSleepTime:
+            time.sleep(settings.gilSleepTime)
+        return True
+
     def start(self):
         """Starts event notification with the AT-SPI Registry.  This method
         only returns after 'stop' has been called.
         """
         Accessible.init(self)
-        bonobo.main()
+
+        # We'll try our own main loop to help debug things.  Code borrowed
+        # "The Whole PyGtk FAQ": http://www.async.com.br/faq/pygtk/
+        #
+        if settings.useBonoboMain:
+            debug.println(debug.LEVEL_CONFIGURATION,
+                          "atspi.start: using bonobo.main; "
+                          + "gilSleepTime=%f" % settings.gilSleepTime)
+            if settings.gilSleepTime:
+                gobject.idle_add(self.__blockPreventor)
+            bonobo.main()
+        else:
+            debug.println(debug.LEVEL_CONFIGURATION,
+                          "atspi.start: using our custom main loop; "
+                          + "gilSleepTime=%f" % settings.gilSleepTime)
+            self.running = True
+            context = gobject.MainLoop().get_context()
+            while self.running:
+                if settings.gilSleepTime:
+                    time.sleep(settings.gilSleepTime)
+                context.iteration(False)
 
     def stop(self):
         """Unregisters any event or keystroke listeners registered with
@@ -136,7 +185,10 @@ class Registry:
         Accessible.shutdown(self)
         for listener in (self.__listeners + self.__keystrokeListeners):
             listener.deregister()
-        bonobo.main_quit()
+        if settings.useBonoboMain:
+            bonobo.main_quit()
+        else:
+            self.running = False
 
     def registerEventListener(self, callback, eventType):
         """Registers the given eventType and callback with the Registry.
@@ -226,11 +278,18 @@ class EventListener(Accessibility__POA.EventListener):
         self.__registered = False
 
     def notifyEvent(self, event):
+        if settings.timeoutCallback and (settings.timeoutTime > 0):
+            signal.signal(signal.SIGALRM, settings.timeoutCallback)
+            signal.alarm(settings.timeoutTime)
+
         try:
             self.callback(event)
         except:
             debug.printException(debug.LEVEL_WARNING)
 
+        if settings.timeoutCallback and (settings.timeoutTime > 0):
+            signal.alarm(0)
+
     def __del__(self):
         self.deregister()
 
@@ -305,11 +364,20 @@ class KeystrokeListener(Accessibility__POA.DeviceEventListener):
 
         Returns True if the event has been consumed.
         """
+        if settings.timeoutCallback and (settings.timeoutTime > 0):
+            signal.signal(signal.SIGALRM, settings.timeoutCallback)
+            signal.alarm(settings.timeoutTime)
+
         try:
-            return self.callback(event)
+            consumed = self.callback(event)
         except:
             debug.printException(debug.LEVEL_WARNING)
-            return False
+            consumed = False
+
+        if settings.timeoutCallback and (settings.timeoutTime > 0):
+            signal.alarm(0)
+
+        return consumed
 
     def __del__(self):
         self.deregister()
@@ -503,11 +571,15 @@ class Accessible:
                 "         Returning Python Accessible.")
             return acc
 
-        # Comment these two lines out to eliminate any local caching
-        # of Accessible objects.
+        # [[[TODO: WDW - the AT-SPI appears to give us a different
+        # accessible when we repeatedly ask for the same child of a
+        # parent that manages its descendants.  So...we probably
+        # shouldn't cache those kind of children because we're likely
+        # to cause a memory leak. Logged as bugzilla bug 319675.]]]
         #
-        #obj = Accessible(acc)
-        #return obj
+        if not settings.cacheAccessibles:
+            obj = Accessible(acc)
+            return obj
 
         if Accessible._cache.has_key(acc):
             obj = Accessible._cache[acc]
@@ -584,11 +656,11 @@ class Accessible:
         try:
             self.accessible = acc._narrow(Accessibility.Application)
             try:
-                self.toolkitName = application.toolkitName
+                self.toolkitName = self.accessible.toolkitName
             except:
                 self.toolkitName = None
             try:
-                self.version = application.version
+                self.version = self.accessible.version
             except:
                 self.version = None
         except:
@@ -601,17 +673,9 @@ class Accessible:
                               + " NOT GIVEN AN ACCESSIBLE!")
                 self.accessible = None
 
+        # Save a reference to the AT-SPI object.
+        #
         if self.accessible:
-            # [[[TODO: WDW - the AT-SPI appears to give us a
-            # different accessible when we repeatedly ask for the
-            # same child of a parent that manages its descendants.
-            # So...we probably shouldn't cache those kind of
-            # children because we're likely to cause a memory
-            # leak. Logged as bugzilla bug 319675.]]]
-            #
-            # Save a reference to the AT-SPI object, and also save this
-            # new object away in the cache.
-            #
             try:
                 self.accessible.ref()
                 self._acc = acc
@@ -808,7 +872,7 @@ class Accessible:
 
         role = self.accessible.getRoleName()
 
-        # [[[WDW - HACK to coalesce menu items with children into
+        # [[[TODO: HACK to coalesce menu items with children into
         # menus.  The menu demo in gtk-demo does this, and one
         # might view that as an edge case.  But, in
         # gnome-terminal, "Terminal" -> "Set Character Encoding"
@@ -825,7 +889,26 @@ class Accessible:
             and (self.childCount > 0):
                 role = rolenames.ROLE_MENU
 
-        if role and settings.cacheValues:
+        # [[[TODO: HACK because Java gives us radio button role and
+        # check box role for menu item objects, instead of giving us
+        # radio menu item role and check menu item role (see SwingSet
+        # menus).]]]
+        #
+        if (self.parent) and (self.parent.role == rolenames.ROLE_MENU):
+            if (role == rolenames.ROLE_RADIO_BUTTON):
+                role = rolenames.ROLE_RADIO_MENU_ITEM
+            elif (role == rolenames.ROLE_CHECK_BOX):
+                role = rolenames.ROLE_CHECK_MENU_ITEM
+
+        # [[[TODO: HACK because we sometimes get an object with an
+        # unknown role but it's role changes later on and we are not
+        # notified.  An example of this is gnome-terminal.  So...to
+        # help with this, we will not cache the role if it is unknown.
+        # See http://bugzilla.gnome.org/show_bug.cgi?id=344218 for
+        # more info.]]]
+        #
+        if role and settings.cacheValues \
+            and (role != rolenames.ROLE_UNKNOWN):
             self.role = role
 
         return role
@@ -849,9 +932,15 @@ class Accessible:
         sequence of Accessible StateTypes.
         """
 
-        stateSet = self.accessible.getState()
+        try:
+            stateSet = self.accessible.getState()
+        except:
+            stateSet = None
         if stateSet:
-            state = stateSet._narrow(Accessibility.StateSet).getStates()
+            try:
+                state = stateSet._narrow(Accessibility.StateSet).getStates()
+            except:
+                state = []
         else:
             state = []
 
@@ -872,7 +961,10 @@ class Accessible:
         relationSet = self.accessible.getRelationSet()
 
         for relation in relationSet:
-            relations.append(relation._narrow(Accessibility.Relation))
+            try:
+                relations.append(relation._narrow(Accessibility.Relation))
+            except:
+                pass
 
         if settings.cacheValues:
             self.relations = relations
@@ -969,7 +1061,10 @@ class Accessible:
         action = self.accessible.queryInterface("IDL:Accessibility/Action:1.0")
 
         if action:
-            action = action._narrow(Accessibility.Action)
+            try:
+                action = action._narrow(Accessibility.Action)
+            except:
+                action = None
 
         if action and settings.cacheValues:
             self.action = action
@@ -986,7 +1081,10 @@ class Accessible:
             "IDL:Accessibility/Component:1.0")
 
         if component:
-            component = component._narrow(Accessibility.Component)
+            try:
+                component = component._narrow(Accessibility.Component)
+            except:
+                component = None
 
         if component and settings.cacheValues:
             self.component = component
@@ -1003,7 +1101,10 @@ class Accessible:
             "IDL:Accessibility/Hypertext:1.0")
 
         if hypertext:
-            hypertext = hypertext._narrow(Accessibility.Hypertext)
+            try:
+                hypertext = hypertext._narrow(Accessibility.Hypertext)
+            except:
+                hypertext = None
 
         if hypertext and settings.cacheValues:
             self.hypertext = hypertext
@@ -1020,7 +1121,10 @@ class Accessible:
             "IDL:Accessibility/Image:1.0")
 
         if image:
-            image = image._narrow(Accessibility.Image)
+            try:
+                image = image._narrow(Accessibility.Image)
+            except:
+                image = None
 
         if image and settings.cacheValues:
             self.image = image
@@ -1037,7 +1141,10 @@ class Accessible:
             "IDL:Accessibility/Selection:1.0")
 
         if selection:
-            selection = selection._narrow(Accessibility.Selection)
+            try:
+                selection = selection._narrow(Accessibility.Selection)
+            except:
+                selection = None
 
         if selection and settings.cacheValues:
             self.selection = selection
@@ -1053,7 +1160,10 @@ class Accessible:
         table = self.accessible.queryInterface("IDL:Accessibility/Table:1.0")
 
         if table:
-            table = table._narrow(Accessibility.Table)
+            try:
+                table = table._narrow(Accessibility.Table)
+            except:
+                table = None
 
         if table and settings.cacheValues:
             self.table = table
@@ -1069,7 +1179,10 @@ class Accessible:
         text = self.accessible.queryInterface("IDL:Accessibility/Text:1.0")
 
         if text:
-            text = text._narrow(Accessibility.Text)
+            try:
+                text = text._narrow(Accessibility.Text)
+            except:
+                text = None
 
         if text and settings.cacheValues:
             self.text = text
@@ -1085,7 +1198,10 @@ class Accessible:
         value = self.accessible.queryInterface("IDL:Accessibility/Value:1.0")
 
         if value:
-            value = value._narrow(Accessibility.Value)
+            try:
+                value = value._narrow(Accessibility.Value)
+            except:
+                value = None
 
         if value and settings.cacheValues:
             self.value = value
@@ -1154,8 +1270,8 @@ class Accessible:
         Arguments:
         - index: an integer specifying which child to obtain
 
-        Returns the child at the given index.  [[[TODO: WDW - what to do
-        when an index out of bounds occurs?  Logged as bugzilla bug 319738.]]]
+        Returns the child at the given index or raise an exception if the
+        index is out of bounds or the child is invalid.
         """
 
         # [[[TODO: WDW - the AT-SPI appears to give us a different accessible
@@ -1166,12 +1282,17 @@ class Accessible:
         # Save away details we now know about this child
         #
         newChild = None
-        accChild = self.accessible.getChildAtIndex(index)
-        if accChild:
-            newChild = Accessible.makeAccessible(accChild)
-            newChild.index = index
-            newChild.parent = self
-            newChild.app = self.app
+        if index >= 0 and index < self.accessible.childCount:
+            accChild = self.accessible.getChildAtIndex(index)
+            if accChild:
+                newChild = Accessible.makeAccessible(accChild)
+                newChild.index = index
+                newChild.parent = self
+                newChild.app = self.app
+
+        if not newChild:
+            raise Exception("Invalid child at index: %d" % index)
+
         return newChild
 
     def toString(self, indent="", includeApp=True):
@@ -1201,381 +1322,6 @@ class Accessible:
 
         return string
 
-########################################################################
-#                                                                      #
-# Utility methods.                                                     #
-#                                                                      #
-########################################################################
-
-def getObjects(root):
-    """Returns a list of all objects under the given root.  Objects
-    are returned in no particular order - this function does a simple
-    tree traversal, ignoring the children of objects which report the
-    MANAGES_DESCENDANTS state is active.
-
-    Arguments:
-    - root: the Accessible object to traverse
-
-    Returns: a list of all objects under the specified object
-    """
-
-    # The list of object we'll return
-    #
-    objlist = []
-
-    # Start at the first child of the given object
-    #
-    if root.childCount <= 0:
-        return objlist
-
-    for i in range(0, root.childCount):
-        debug.println(debug.LEVEL_FINEST,
-                      "atspi.getObjects looking at child %d" % i)
-        child = root.child(i)
-        if child:
-            objlist.append(child)
-            state = child.state
-            if (state.count(Accessibility.STATE_MANAGES_DESCENDANTS) == 0) \
-                   and (child.childCount > 0):
-                objlist.extend(getObjects(child))
-
-    return objlist
-
-def findByRole(root, role):
-    """Returns a list of all objects of a specific role beneath the
-    given root.  [[[TODO: MM - This is very inefficient - this should
-    do it's own traversal and not add objects to the list that aren't
-    of the specified role.  Instead it uses the traversal from
-    getObjects and then deletes objects from the list that aren't of
-    the specified role.  Logged as bugzilla bug 319740.]]]
-
-    Arguments:
-    - root the Accessible object to traverse
-    - role the string describing the Accessible role of the object
-
-    Returns a list of descendants of the root that are of the given role.
-    """
-
-    objlist = []
-    allobjs = getObjects(root)
-    for o in allobjs:
-        if o.role == role:
-            objlist.append(o)
-    return objlist
-
-def findByName(root, name):
-    """Returns a list of all objects beneath the specified object with
-    the specified name.  [[[TODO: MM - This is very inefficient - this
-    should do it's own traversal and not add objects to the list that
-    don't have the specified name.  Instead it uses the traversal from
-    getObjects and then deletes objects from the list that don't have
-    the specified name. Logged as bugzilla bug 319740.]]]
-
-    Arguments:
-    - root the Accessible object to traverse
-    - name the string containing the Accessible name of the object
-
-    Returns a list of descendants of the root that are of the given role.
-    """
-
-    objlist = []
-    objects = getObjects(root)
-
-    # Create a new list containing only those objects with the
-    # specified name
-    #
-    for o in objects:
-        if o.name == name:
-            objlist.append(o)
-    return objlist
-
-def findUnrelatedLabels(root):
-    """Returns a list containing all the unrelated (i.e., have no
-    relations to anything and are not a fundamental element of a
-    more atomic component like a combo box) labels under the given
-    root.  Note that the labels must also be showing on the display.
-
-    Arguments:
-    - root the Accessible object to traverse
-
-    Returns a list of unrelated labels under the given root.
-    """
-
-    # Find all the labels in the dialog
-    #
-    allLabels = findByRole(root, rolenames.ROLE_LABEL)
-
-    # add the names of only those labels which are not associated with
-    # other objects (i.e., empty relation sets).
-    #
-    # [[[WDW - HACK: In addition, do not grab free labels whose
-    # parents are push buttons because push buttons can have labels as
-    # children.]]]
-    #
-    # [[[WDW - HACK: panels with labelled borders will have a child
-    # label that does not have its relation set.  So...we check to see
-    # if the panel's name is the same as the label's name.  If so, we
-    # ignore the label.]]]
-    #
-    unrelatedLabels = []
-
-    for label in allLabels:
-        relations = label.relations
-        if len(relations) == 0:
-            parent = label.parent
-            if parent and (parent.role == rolenames.ROLE_PUSH_BUTTON):
-                pass
-            elif parent and (parent.role == rolenames.ROLE_PANEL) \
-               and (parent.name == label.name):
-                pass
-            elif label.state.count(Accessibility.STATE_SHOWING):
-                unrelatedLabels.append(label)
-
-    # Now sort the labels based on their geographic position, top to
-    # bottom, left to right.  This is a very inefficient sort, but the
-    # assumption here is that there will not be a lot of labels to
-    # worry about.
-    #
-    sortedLabels = []
-    for label in unrelatedLabels:
-        index = 0
-        for sortedLabel in sortedLabels:
-            if (label.extents.y > sortedLabel.extents.y) \
-               or ((label.extents.y == sortedLabel.extents.y) \
-                   and (label.extents.x > sortedLabel.extents.x)):
-                index += 1
-            else:
-                break
-        sortedLabels.insert(index, label)
-
-    #return unrelatedLabels
-    return sortedLabels
-
-def getFrame(obj):
-    """Returns the frame containing this object, or None if this object
-    is not inside a frame.
-
-    Arguments:
-    - obj: the Accessible object
-    """
-
-    debug.println(debug.LEVEL_FINEST,
-                  "Finding frame for source.name="
-                  + obj.accessibleNameToString())
-
-    while obj \
-          and (obj != obj.parent) \
-          and (obj.role != rolenames.ROLE_FRAME):
-        obj = obj.parent
-        debug.println(debug.LEVEL_FINEST, "--> obj.name="
-                      + obj.accessibleNameToString())
-
-    if obj and (obj.role == rolenames.ROLE_FRAME):
-        pass
-    else:
-        obj = None
-
-    return obj
-
-def getTextLineAtCaret(obj):
-    """Gets the line of text where the caret is.
-
-    Argument:
-    - obj: an Accessible object that implements the AccessibleText
-           interface
-
-    Returns the line of text where the caret is.
-    """
-
-    # Get the the AccessibleText interrface
-    #
-    text = obj.text
-    if not text:
-        return ["", 0, 0]
-
-    # Get the line containing the caret
-    #
-    offset = text.caretOffset
-    line = text.getTextAtOffset(offset,
-                                Accessibility.TEXT_BOUNDARY_LINE_START)
-
-    # Line is actually a list of objects-- the first is the actual
-    # text of the line, the second is the start offset, and the third
-    # is the end offset.  Sometimes we get the trailing line-feed-- remove it
-    #
-    if line[0][-1:] == "\n":
-        content = line[0][:-1]
-    else:
-        content = line[0]
-
-    return [content, offset, line[1]]
-
-def getNodeLevel(obj):
-    """Determines the node level of this object if it is in a tree
-    relation, with 0 being the top level node.  If this object is
-    not in a tree relation, then -1 will be returned.
-
-    Arguments:
-    -obj: the Accessible object
-    """
-
-    if not obj:
-        return -1
-
-    level = -1
-    node = obj
-    done = False
-    while not done:
-        relations = node.relations
-        node = None
-        for relation in relations:
-            if relation.getRelationType() \
-                   == Accessibility.RELATION_NODE_CHILD_OF:
-                level += 1
-                node = Accessible.makeAccessible(relation.getTarget(0))
-                break
-        done = not node
-        debug.println(debug.LEVEL_FINEST, "atspi.getNodeLevel %d" % level)
-
-    return level
-
-def getAcceleratorAndShortcut(obj):
-    """Gets the accelerator string (and possibly shortcut) for the given
-    object.
-
-    Arguments:
-    - obj: the Accessible object
-
-    A list containing the accelerator and shortcut for the given object,
-    where the first element is the accelerator and the second element is
-    the shortcut.
-    """
-
-    action = obj.action
-
-    if not action:
-        return ["", ""]
-
-    # [[[TODO: WDW - assumes the first keybinding is all that we care about.
-    # Logged as bugzilla bug 319741.]]]
-    #
-    bindingStrings = action.getKeyBinding(0).split(';')
-
-    # [[[TODO: WDW - assumes menu items have three bindings.  Logged as
-    # bugzilla bug 319741.]]]
-    #
-    if len(bindingStrings) == 3:
-        #mnemonic       = bindingStrings[0]
-        fullShortcut   = bindingStrings[1]
-        accelerator    = bindingStrings[2]
-    elif len(bindingStrings) > 0:
-        fullShortcut   = bindingStrings[0]
-        accelerator    = ""
-    else:
-        fullShortcut   = ""
-        accelerator    = ""
-
-    fullShortcut = fullShortcut.replace("<","")
-    fullShortcut = fullShortcut.replace(">"," ")
-    fullShortcut = fullShortcut.replace(":"," ")
-
-    accelerator  = accelerator.replace("<","")
-    accelerator  = accelerator.replace(">"," ")
-
-    return [accelerator, fullShortcut]
-
-def getKnownApplications():
-    """Retrieves the list of currently running apps for the desktop
-    as a list of Accessible objects.
-    """
-
-    debug.println(debug.LEVEL_FINEST,
-                  "atspi.getKnownApplications...")
-
-    apps = []
-    registry = Registry()
-    for i in range(0, registry.desktop.childCount):
-        try:
-            acc = registry.desktop.getChildAtIndex(i)
-            app = Accessible.makeAccessible(acc)
-            if app:
-                apps.insert(0, app)
-        except:
-            debug.printException(debug.LEVEL_FINEST)
-
-    debug.println(debug.LEVEL_FINEST,
-                  "...orca._buildAppList")
-
-    return apps
-
-########################################################################
-#                                                                      #
-# Debug utilities                                                      #
-#                                                                      #
-########################################################################
-
-def printAncestry(child):
-   """Prints a hierarchical view of a child's ancestry."""
-
-   if not child:
-       return
-
-   ancestorList = [child]
-   parent = child.parent
-   while parent and (parent.parent != parent):
-      ancestorList.insert(0, parent)
-      parent = parent.parent
-
-   indent = ""
-   for ancestor in ancestorList:
-      print ancestor.toString(indent + "+-", False)
-      indent += "  "
-
-def printHierarchy(root, ooi, indent="", onlyShowing=True, omitManaged=True):
-    """Prints the accessible hierarchy of all children
-
-    Arguments:
-    -indent:      Indentation string
-    -root:        Accessible where to start
-    -ooi:         Accessible object of interest
-    -onlyShowing: If True, only show children painted on the screen
-    -omitManaged: If True, omit children that are managed descendants
-    """
-
-    if not root:
-        return
-
-    if root == ooi:
-       print root.toString(indent + "(*)", False)
-    else:
-       print root.toString(indent + "+-", False)
-
-    rootManagesDescendants = root.state.count(\
-        Accessibility.STATE_MANAGES_DESCENDANTS)
-
-    for i in range(0, root.childCount):
-        child = root.child(i)
-        if child == root:
-            print indent + "  " + "WARNING CHILD == PARENT!!!"
-        elif not child:
-            print indent + "  " + "WARNING CHILD IS NONE!!!"
-        elif child.parent != root:
-            print indent + "  " + "WARNING CHILD.PARENT != PARENT!!!"
-        else:
-            paint = (not onlyShowing) \
-                    or (onlyShowing \
-                        and child.state.count(Accessibility.STATE_SHOWING))
-            paint = paint \
-                    and ((not omitManaged) \
-                         or (omitManaged and not rootManagesDescendants))
-
-            if paint:
-               printHierarchy(child,
-                              ooi,
-                              indent + "  ",
-                              onlyShowing,
-                              omitManaged)
-
 ########################################################################
 #                                                                      #
 # Testing functions.                                                   #
@@ -1688,7 +1434,6 @@ def __test():
     registry.start()
 
 if __name__ == "__main__":
-    import signal
     signal.signal(signal.SIGINT, __shutdownAndExit)
     signal.signal(signal.SIGQUIT, __shutdownAndExit)
     __test()
