@@ -17,8 +17,13 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-"""Provides support for synthesizing keyboard and mouse events.
-"""
+"""Provides support for synthesizing keyboard and mouse events."""
+
+__id__        = "$Id: eventsynthesizer.py,v 1.7 2006/06/10 00:54:47 wwalker Exp $"
+__version__   = "$Revision: 1.7 $"
+__date__      = "$Date: 2006/06/10 00:54:47 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
 
 import atspi
 import debug
