@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py,v 1.80 2006/07/24 01:32:30 wwalker Exp $"
-__version__   = "$Revision: 1.80 $"
-__date__      = "$Date: 2006/07/24 01:32:30 $"
+__id__        = "$Id: focus_tracking_presenter.py,v 1.85 2006/08/10 11:53:22 wwalker Exp $"
+__version__   = "$Revision: 1.85 $"
+__date__      = "$Date: 2006/08/10 11:53:22 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -35,7 +35,7 @@ import braille
 import default
 import debug
 import input_event
-import orca
+import orca_state
 import presentation_manager
 import settings
 import speech
@@ -67,6 +67,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         #
         self.registry        = atspi.Registry()
         self._knownScripts   = {}
+        self._activeScript   = None
         self._eventQueue     = Queue.Queue(0)
         self._gidleId        = 0
         self._gidleLock      = threading.Lock()
@@ -181,7 +182,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                                       "...found %s.py" % name)
                         break
                     except ImportError:
-                        debug.printException(debug.LEVEL_ALL)
                         debug.println(debug.LEVEL_FINEST,
                                       "...could not find %s.py" % name)
                     except:
@@ -221,7 +221,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 debug.println(debug.LEVEL_FINE,
                               "...found %s.py" % name)
             except ImportError:
-                debug.printException(debug.LEVEL_ALL)
                 debug.println(
                     debug.LEVEL_FINE,
                     "...could not find %s.py" % app.toolkitName)
@@ -396,7 +395,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # as bugzilla bug 319777.]]]
         #
         retryCount = 0
-        oldLocusOfFocus = orca.locusOfFocus
+        oldLocusOfFocus = orca_state.locusOfFocus
         while retryCount <= settings.commFailureAttemptLimit:
             try:
                 if event.type == "window:activate":
@@ -424,7 +423,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     # of focus changed during our last attempt at
                     # handling this event.
                     #
-                    orca.locusOfFocus = oldLocusOfFocus
+                    orca_state.locusOfFocus = oldLocusOfFocus
                     debug.println(debug.LEVEL_WARNING,
                                   "  TRYING AGAIN (%d)" % retryCount)
                     time.sleep(settings.commFailureWaitTime)
@@ -534,7 +533,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 debug.println(debug.LEVEL_ALL,
                               "           ...put complete")
             if not self._gidleId:
-                time.sleep(0.0001) # Attempt to sidestep GIL
+                if settings.gilSleepTime:
+                    time.sleep(settings.gilSleepTime)
                 self._gidleId = gobject.idle_add(self._dequeueEvent)
 
             if settings.debugEventQueue:
@@ -558,13 +558,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         #    print "Quitting Orca."
         #    orca.shutdown()
 
-    def _timeout(self):
-        """Timer that will be called if we time out while trying to perform
-        an operation."""
-        debug.println(debug.LEVEL_SEVERE,
-                      "TIMEOUT: Looks like something has hung. Quitting Orca.")
-        orca.shutdown()
-
     def _dequeueEvent(self):
         """Handles all events destined for scripts.  Called by the GTK
         idle thread.
@@ -614,13 +607,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 debug.println(debug.LEVEL_FINEST, "DEQUEUED EVENT %s <----------" \
                               % event.type)
 
-                # [[[TODO: WDW - the timer stuff is an experiment to see if
-                # we can recover from hangs.  It's only experimental, so it's
-                # commented out for now.]]]
-                #
-                #timer = threading.Timer(5.0, self._timeout)
-                #timer.start()
-
                 if (not debug.eventDebugFilter) \
                     or (debug.eventDebugFilter \
                         and debug.eventDebugFilter.match(event.type)):
@@ -635,21 +621,21 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                                   "^^^^^ PROCESS OBJECT EVENT %s ^^^^^\n" \
                                   % event.type)
 
-                #timer.cancel()
-                #del timer
-
             # [[[TODO: HACK - it would seem logical to only do this if we
             # discover the queue is empty, but this inroduces a hang for
             # some reason if done inside an acquire/release block for a
             # lock.  So...we do it here.]]]
             #
-            noFocus = (not self._activeScript) or ((not orca.locusOfFocus) \
-                      and (self.noFocusTimestamp != orca.noFocusTimeStamp))
+            noFocus = (not self._activeScript) \
+                      or ((not orca_state.locusOfFocus) \
+                          and (self.noFocusTimestamp \
+                               != orca_state.noFocusTimestamp))
 
             self._gidleLock.acquire()
             if self._eventQueue.empty():
                 if noFocus:
-                    time.sleep(0.0001) # Attempt to sidestep GIL
+                    if settings.gilSleepTime:
+                        time.sleep(settings.gilSleepTime)
                     message = _("No focus")
                     if settings.brailleVerbosityLevel == \
                         settings.VERBOSITY_LEVEL_VERBOSE:
@@ -657,7 +643,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     if settings.speechVerbosityLevel == \
                         settings.VERBOSITY_LEVEL_VERBOSE:
                         speech.speak(message)
-                    self.noFocusTimestamp = orca.noFocusTimeStamp
+                    self.noFocusTimestamp = orca_state.noFocusTimestamp
                 self._gidleId = 0
                 rerun = False # destroy and don't call again
             self._gidleLock.release()
