@@ -27,9 +27,9 @@ The following script specific key sequences are supported:
   Insert-[1-9]  -  Speak and braille a previous chat room message.
 """
 
-__id__        = "$Id: gaim.py,v 1.19 2006/07/22 03:04:18 wwalker Exp $"
-__version__   = "$Revision: 1.19 $"
-__date__      = "$Date: 2006/07/22 03:04:18 $"
+__id__        = "$Id: gaim.py,v 1.21 2006/07/28 17:33:18 wwalker Exp $"
+__version__   = "$Revision: 1.21 $"
+__date__      = "$Date: 2006/07/28 17:33:18 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -39,8 +39,8 @@ import orca.debug as debug
 import orca.default as default
 import orca.input_event as input_event
 import orca.keybindings as keybindings
-import orca.orca as orca
 import orca.rolenames as rolenames
+import orca.settings as settings
 import orca.speech as speech
 import orca.util as util
 
@@ -166,8 +166,8 @@ class Script(default.Script):
         keyBindings.add(
             keybindings.KeyBinding(
                 "h",
-                1 << orca.MODIFIER_ORCA,
-                1 << orca.MODIFIER_ORCA,
+                1 << settings.MODIFIER_ORCA,
+                1 << settings.MODIFIER_ORCA,
                 self.togglePrefixHandler))
 
         messageKeys = [ "1", "2", "3", "4", "5", "6", "7", "8", "9" ]
@@ -175,8 +175,8 @@ class Script(default.Script):
             keyBindings.add(
                 keybindings.KeyBinding(
                     messageKeys[i],
-                    1 << orca.MODIFIER_ORCA,
-                    1 << orca.MODIFIER_ORCA,
+                    1 << settings.MODIFIER_ORCA,
+                    1 << settings.MODIFIER_ORCA,
                     self.readPreviousMessageHandler))
 
         return keyBindings
@@ -321,7 +321,7 @@ class Script(default.Script):
 
             message = event.source.text.getText(event.detail1,
                                                 event.detail1 + event.detail2)
-            if message[0] == "\n":
+            if message and message[0] == "\n":
                 message = message[1:]
 
             chatRoomName = util.getDisplayedText(chatRoomTab)
