@@ -19,9 +19,9 @@
 
 """Custom script for gedit."""
 
-__id__        = "$Id: gedit.py,v 1.15 2006/06/10 00:54:48 wwalker Exp $"
-__version__   = "$Revision: 1.15 $"
-__date__      = "$Date: 2006/06/10 00:54:48 $"
+__id__        = "$Id: gedit.py,v 1.18 2006/08/28 22:41:55 richb Exp $"
+__version__   = "$Revision: 1.18 $"
+__date__      = "$Date: 2006/08/28 22:41:55 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -29,7 +29,7 @@ import orca.debug as debug
 import orca.atspi as atspi
 import orca.braille as braille
 import orca.default as default
-import orca.orca as orca
+import orca.orca_state as orca_state
 import orca.rolenames as rolenames
 import orca.speech as speech
 import orca.speechgenerator as speechgenerator
@@ -123,7 +123,7 @@ class Script(default.Script):
 
         # Braille the default action for this component.
         #
-        self.updateBraille(orca.locusOfFocus)
+        self.updateBraille(orca_state.locusOfFocus)
 
         # Look for the label containing the misspelled word.
         # There will be three labels in the top panel in the Check
@@ -314,3 +314,24 @@ class Script(default.Script):
 
         # Pass the event onto the parent class to be handled in the default way.
         default.Script.onNameChanged(self, event)
+
+    def onStateChanged(self, event):
+        """Called whenever an object's state changes.
+
+        Arguments:
+        - event: the Event
+        """
+
+        # Sometimes an object will tell us it is focused this
+        # way versus issuing a focus event.  GEdit's edit area,
+        # for example, will do this: when you use metacity's
+        # window menu to do things like maximize or unmaximize
+        # a window, you will only get a state-changed event
+        # from the text area when it regains focus.
+        # (See bug #350854 for more details).
+        #
+        if (event.type == "object:state-changed:focused") \
+           and (event.detail1):
+            orca.setLocusOfFocus(event, event.source)
+
+        default.Script.onStateChanged(self, event)
