@@ -17,12 +17,20 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-import orca.atspi as atspi
+"""Custom script for metacity."""
+
+__id__        = "$Id: metacity.py,v 1.15 2006/06/10 00:54:48 wwalker Exp $"
+__version__   = "$Revision: 1.15 $"
+__date__      = "$Date: 2006/06/10 00:54:48 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
+
 import orca.braille as braille
 import orca.default as default
 import orca.rolenames as rolenames
 import orca.orca as orca
 import orca.speech as speech
+import orca.util as util
 
 from orca.orca_i18n import _
 
@@ -42,48 +50,48 @@ class Script(default.Script):
         """
         default.Script.__init__(self, app)
 
-    def getListeners(self):
-        """Sets up the AT-SPI event listeners for this script.
-        """
-
-        listeners = default.Script.getListeners(self)
-        
-        listeners["object:property-change:accessible-name"] = \
-            self.onNameChanged
-        
-        listeners["object:state-changed:visible"] = \
-            self.onVisibilityChanged
-
-        return listeners
-    
     def onNameChanged(self, event):
         """The status bar in metacity tells us what toplevel window will be
-        activated when tab is released.
+        activated when tab is released.  We will key off the name changed
+        event to determine when to say something, as it seems to be the
+        more reliable event.
 
         Arguments:
         - event: the name changed Event
         """
 
-        # If it's not the statusbar's name changing, ignore it
+        # Do the default thing if this is not the status bar.
         #
         if event.source.role != rolenames.ROLE_STATUSBAR:
             default.Script.onNameChanged(self, event)
             return
 
+        # Let's make sure the name really changed.  Metacity seems
+        # to like to give us multiple name changed events.
+        #
+        if event.source.__dict__.has_key("oldName"):
+            oldName = event.source.oldName
+        else:
+            oldName = None
+
+        name = event.source.name
+        if name == oldName:
+            return
+        else:
+            event.source.oldName = name
+
         # We have to stop speech, as Metacity has a key grab and we're not
         # getting keys
         #
         speech.stop()
 
-        name = event.source.name
-
         # Do we know about this window?  Traverse through our list of apps
         # and go through the toplevel windows in each to see if we know
         # about this one.  If we do, it's accessible.  If we don't, it is
         # not.
         #
         found = False
-        for app in atspi.getKnownApplications():
+        for app in util.getKnownApplications():
             i = 0
             while i < app.childCount:
                 win = app.child(i)
@@ -107,20 +115,18 @@ class Script(default.Script):
         braille.displayMessage(text)
         speech.speak(text)
 
-    def onVisibilityChanged(self, event):
+    def onStateChanged(self, event):
         """The status bar in metacity tells us what toplevel window will be
         activated when tab is released.
 
         Arguments:
-        - event: the object:state-changed:visible Event
+        - event: the object:state-changed: Event
         """
 
-        # If it's not the statusbar's name changing, ignore it
+        # Ignore changes on the status bar.  We handle them in onNameChanged.
         #
         if event.source.role != rolenames.ROLE_STATUSBAR:
-            return
-
-        orca.setLocusOfFocus(event, event.source)
+            default.Script.onStateChanged(self, event)
 
     def onTextInserted(self, event):
         """Called whenever text is inserted into an object.
@@ -128,6 +134,9 @@ class Script(default.Script):
         Arguments:
         - event: the Event
         """
+
+        # Ignore changes on the status bar.  We handle them in onNameChanged.
+        #
         if event.source.role != rolenames.ROLE_STATUSBAR:
             default.Script.onTextInserted(self, event)
 
@@ -137,6 +146,9 @@ class Script(default.Script):
         Arguments:
         - event: the Event
         """
+
+        # Ignore changes on the status bar.  We handle them in onNameChanged.
+        #
         if event.source.role != rolenames.ROLE_STATUSBAR:
             default.Script.onTextDeleted(self, event)
 
@@ -146,5 +158,8 @@ class Script(default.Script):
         Arguments:
         - event: the Event
         """
+
+        # Ignore changes on the status bar.  We handle them in onNameChanged.
+        #
         if event.source.role != rolenames.ROLE_STATUSBAR:
             default.Script.onCaretMoved(self, event)
